<?php
/**
 * @file
 * Webform module serial component.
 */

/**
 * Implements _webform_defaults_component().
 */
function _webform_defaults_serial() {
  return array(
    'name' => '',
    'form_key' => NULL,
    'pid' => 0,
    'weight' => 0,
    'value' => '',
    'mandatory' => 0,
    'extra' => array(
      'initial' => 1,
      'increment' => 1,
      'private' => TRUE,
    ),
  );
}

/**
 * Implements _webform_edit_component().
 */
function _webform_edit_serial($component) {
  drupal_set_message(t('Please do not disable the <em>Private</em> checkbox.'), 'warning');

  $form = array();
  $form['extra']['initial'] = array(
    '#type' => 'textfield',
    '#title' => t('Initial/next value'),
    '#default_value' => empty($component['extra']['initial']) ? 1 : $component['extra']['initial'],
    '#description' => t('The value of the next serial number. This is usually 1 when you start and will go up with each form submission.'),
    '#size' => 8,
    '#maxlength' => 1024,
    '#weight' => 0,
    '#element_validate' => array('_webform_edit_serial_validate'),
  );
  $form['extra']['increment'] = array(
    '#type' => 'textfield',
    '#title' => t('Increment value'),
    '#default_value' => empty($component['extra']['increment']) ? 1 : $component['extra']['increment'],
    '#description' => t('The increment value of the field. Each time the form is submitted, add this many to the serial field.'),
    '#size' => 8,
    '#maxlength' => 1024,
    '#weight' => 0,
    '#element_validate' => array('_webform_edit_serial_validate'),
  );
  return $form;
}

/**
 * Validation of serial edit form items.
 */
function _webform_edit_serial_validate($element, &$form_state) {
  $values = $form_state['values']['extra'];
  switch ($element['#name']) {
    case 'extra[initial]':
      if (!is_numeric($values['initial'])) {
        form_error($element, t('Initial value needs to be a number.'));
      }
      break;

    case 'extra[increment]':
      if (empty($values['increment']) || !is_numeric($values['increment'])) {
        form_error($element, t('Increment value needs to be a number and may not be zero.'));
      }
      break;
  }
}

/**
 * Implements _webform_render_component().
 */
function _webform_render_serial($component, $value, $format = 'text') {
  return array(
    '#title' => $component['name'],
    '#weight' => $component['weight'],
    '#format' => 'text',
    '#markup' => $value[0],
    '#webform_component' => $component,
    '#field_suffix' => NULL,
    '#field_prefix' => NULL,
  );
}

/**
 * Implements _webform_display_component().
 */
function _webform_display_serial($component, $value, $format = 'text') {
  return array(
    '#title' => $component['name'],
    '#weight' => $component['weight'],
    '#theme' => 'webform_display_textfield',
    '#theme_wrappers' => $format == 'html' ? array('webform_element') : array('webform_element_text'),
    '#format' => $format,
    '#value' => isset($value[0]) ? $value[0] : '',
    '#translatable' => array('title'),
    '#field_suffix' => NULL,
    '#field_prefix' => NULL,
  );
}

/**
 * Implements _webform_analysis_component().
 */
function _webform_analysis_serial($component, $sids = array()) {
  $query = db_select('webform_submitted_data', 'wsd', array('fetch' => PDO::FETCH_ASSOC));
  $query->addExpression('MIN(data)', 'minimum');
  $query->condition('nid', $component['nid']);
  $query->condition('cid', $component['cid']);

  if (count($sids)) {
    $query->condition('sid', $sids, 'IN');
  }
  $minimum = $query->execute()->fetchField();

  $query = db_select('webform_submitted_data', 'wsd', array('fetch' => PDO::FETCH_ASSOC));
  $query->addExpression('MAX(CAST(data AS SIGNED))', 'maximum');
  $query->condition('nid', $component['nid']);
  $query->condition('cid', $component['cid']);

  if (count($sids)) {
    $query->condition('sid', $sids, 'IN');
  }
  $maximum = $query->execute()->fetchField();

  $rows[0] = array(t('Lowest serial'), $minimum);
  $rows[1] = array(t('Highest serial'), $maximum);
  return $rows;
}

/**
 * Implements _webform_table_component().
 */
function _webform_table_serial($component, $value) {
  return check_plain(empty($value[0]) ? '' : $value[0]);
}

/**
 * Implements _webform_csv_headers_component().
 */
function _webform_csv_headers_serial($component, $export_options) {
  $header = array();
  $header[0] = '';
  $header[1] = '';
  $header[2] = $component['name'];
  return $header;
}

/**
 * Implements _webform_csv_data_component().
 */
function _webform_csv_data_serial($component, $export_options, $value) {
  return $value;
}

/**
 * Overridable theme function.
 */
function theme_webform_display_serial($variables) {
  $element = $variables['element'];
  return $element['#format'] == 'html' ? check_plain($element['#markup']) : $element['#markup'];
}

/**
 * Implements hook_webform_submission_presave().
 */
function webform_serial_webform_submission_presave($node, &$submission) {
  foreach ($node->webform['components'] as $cid => $component) {
    // If component is a serial number with no value, generate a value.
    if ($component['type'] == 'serial' && empty($submission->data[$cid])) {
      // Calculate the increment (default to 1).
      $increment = empty($component['extra']['increment']) ? 1 : (int) $component['extra']['increment'];

      // Get largest serial number in the database to ensure the new one is larger.
      $query = db_select('webform_submitted_data')
        ->condition('nid', $node->nid)
        ->condition('cid', $cid);
      // CAST() is needed because value is stored as text but must be sorted as a number.
      $query->addExpression('MAX(CAST(data AS signed))');
      $current_max = (int) $query->execute()->fetchField();

      // Use a transaction with SELECT ... FOR UPDATE to lock the row between
      // the SELECT and the UPDATE, ensuring that multiple Webform submissions
      // at the same time do not have duplicate numbers.
      $txn = db_transaction();

      // Get the next serial number as configured in the component.
      $component_extra = db_select('webform_component', 'wc')
        ->forUpdate()
        ->fields('wc', array('extra'))
        ->condition('nid', $node->nid)
        ->condition('cid', $cid)
        ->execute()
        ->fetchField();
      $component_extra = unserialize($component_extra);
      $next_value = empty($component['extra']['initial']) ? 1 : $component['extra']['initial'];

      // If the next value is smaller than the largest value currently in the
      // database, increment the largest value and use that as the next value.
      if ($next_value <= $current_max) {
        $next_value = $current_max + $increment;
      }

      // Increment the next value and save it in the database.
      $component_extra['initial'] = $next_value + $increment;
      db_update('webform_component')
        ->fields(array('extra' => serialize($component_extra)))
        ->condition('nid', $node->nid)
        ->condition('cid', $cid)
        ->execute();

      // Set the serial number in the submission data in a way that is
      // compatible with Webform 3 and Webform 4. If version is not present,
      // such as when working from git, assume Webform 4.
      $value = array($next_value);
      $version = system_get_info('module', 'webform');
      $version = explode('-', $version['version']);
      if (isset($version[1]) && ((int) $version[1] < 4)) {
        $submission->data[$cid]['value'] = $value;
      }
      else {
        $submission->data[$cid] = $value;
      }
    }
  }
}
